RUZ_API_ENDPOINTS = {
    'schedule': "personLessons",
    'groups': "groups",
    'staffOfGroup': "staffOfGroup",
    'staff_of_group': "staffOfGroup",
    'streams': "streams",
    'staffOfStreams': "staffOfStreams",
    'staff_of_streams': "staffOfStreams",
    'lecturers': "lecturers",
    'auditoriums': "auditoriums",
    'typeOfAuditoriums': "typeOfAuditoriums",
    'type_of_auditoriums': "typeOfAuditoriums",
    'kindOfWorks': "kindOfWorks",
    'kind_of_works': "kindOfWorks",
    'buildings': "buildings",
    'faculties': "faculties",
    'chairs': "chairs",
    'subGroups': "subGroups",
    'subgroups': "subGroups",
    'sub_groups': "subGroups"
}

REQUEST_SCHEMA = {
    'schedule': {
        'from_date': str,
        'fromDate': str,
        'to_date': str,
        'toDate': str,
        'receiverType': int,
        'groupOid': int,
        'lecturerOid': int,
        'auditoriumOid': int,
        'studentOid': int,
        'email': str
    },
    'groups': {
        'facultyOid': int,
        'findText': str
    },
    'staffOfGroup': {
        'group_id': int,
        'groupOid': int,
        'findText': str
    },
    'staff_of_group': {
        'group_id': int,
        'groupOid': int,
        'findText': str
    },
    'streams': {'findText': str},
    'staffOfStreams': {
        'streamOid': int,
        'stream_id': int
    },
    'staff_of_streams': {
        'streamOid': int,
        'stream_id': int
    },
    'lecturers': {
        'chairOid': int,
        'findText': str
    },
    'auditoriums': {
        'buildingOid': int,
        'findText': str
    },
    'typeOfAuditoriums': {},
    'type_of_auditoriums': {},
    'kindOfWorks': {},
    'kind_of_works': {},
    'buildings': {'findText': str},
    'faculties': {'findText': str},
    'chairs': {
        'facultyOid': int,
        'findText': str
    },
    'subGroups': {'findText': str},
    'subgroups': {'findText': str},
    'sub_groups': {'findText': str}
}
RESPONSE_SCHEMA = {
    'schedule': [
        {
            'auditorium': str,
            'auditoriumOid': int,
            'beginLesson': str,
            'building': str,
            'date': str,
            'dateOfNest': str,
            'dayOfWeek': int,
            'dayOfWeekString': str,
            'detailInfo': str,
            'discipline': str,
            'disciplineinplan': str,
            'disciplinetypeload': int,
            'endLesson': str,
            'group': None,
            'groupOid': int,
            'isBan': bool,
            'kindOfWork': str,
            'lecturer': str,
            'lecturerOid': int,
            'stream': str,
            'streamOid': int,
            'subGroup': None,
            'subGroupOid': int
        }
    ],
    'schedule2': {
        'Count': int,
        'Lessons': [
            {
                'auditorium': str,
                'auditoriumOid': int,
                'beginLesson': str,
                'building': str,
                'date': str,
                'dateOfNest': str,
                'dayOfWeek': int,
                'dayOfWeekString': str,
                'detailInfo': str,
                'discipline': str,
                'disciplineinplan': str,
                'disciplinetypeload': int,
                'endLesson': str,
                'group': None,
                'groupOid': int,
                'isBan': bool,
                'kindOfWork': str,
                'lecturer': str,
                'lecturerOid': int,
                'stream': str,
                'streamOid': int,
                'subGroup': None,
                'subGroupOid': int
            }
        ],
        'StatusCode': {
            'Code': int,
            'Description': str
        }
    },  # api v2
    'groups': [
        {
            'chairOid': int,
            'course': int,
            'faculty': str,
            'facultyOid': int,
            'formOfEducation': str,
            'groupOid': int,
            'kindEducation': int,
            'number': str,
            'speciality': str
        }
    ],
    'staffOfGroup': [
        {
            'fio': str,
            'shortFIO': str,
            'studentOid': int
        }
    ],
    'streams': [
        {
            'abbr': str,
            'course': str,
            'faculty': str,
            'facultyOid': int,
            'formOfEducation': str,
            'name': str,
            'streamOid': int,
            'yearOfEducation': int
        }
    ],
    'staffOfStreams': [
        {
            "GroupNumber": str,
            "GroupOid": int,
            "SubgroupName": str,
            "SubgroupOid": int
        }
    ],
    'lecturers': [
        {
            'chair': str,
            'chairOid': int,
            'fio': str,
            'lecturerOid': int,
            'shortFIO': str
        }
    ],
    'auditoriums': [
        {
            'auditoriumOid': int,
            'building': str,
            'buildingOid': int,
            'number': str,
            'typeOfAuditorium': str
        }
    ],
    'typeOfAuditoriums': [
        {
            'abbr': str,
            'code': str,
            'name': str,
            'typeOfAuditoriumOid': int
        }
    ],
    'kindOfWorks': [
        {
            'abbr': str,
            'code': None,
            'complexity': int,
            'kindOfWorkOid': int,
            'name': str,
            'unit': str
        }
    ],
    'buildings': [
        {
            'abbr': str,
            'address': str,
            'buildingOid': int,
            'name': str
        }
    ],
    'faculties': [
        {
            'abbr': str,
            'code': None,
            'facultyOid': int,
            'institute': str,
            'name': str
        }
    ],
    'chairs': [
        {
            'abbr': str,
            'chairOid': int,
            'code': str,
            'faculty': str,
            'facultyOid': int,
            'name': str
        }
    ],
    'subGroups': [
        {
            'abbr': str,
            'group': str,
            'groupOid': int,
            'name': str,
            'subGroupOid': int
        }
    ]
}



import os

from ruz.utils.logging import Logger, LEVELS
from ruz.utils.schema import REQUEST_SCHEMA, RESPONSE_SCHEMA, RUZ_API_ENDPOINTS


RUZ_API_URL = "http://92.242.58.221/ruzservice.svc/"
EMAIL_PATTERN = r"\b[a-zA-Z0-9\._-]{2,}@([a-zA-Z]{2,}\.)?[a-zA-Z]{2,}\.ru\b"
EMAIL_DOMAINS = ("hse.ru", "edu.hse.ru")



import pickle
import os


def load_dump(path: str) -> object:
    ''' load object from pickle dump '''
    if os.path.exists(path):
        with open(path, 'rb') as reader:
            return pickle.load(reader)


def save_dump(path: str, obj: object, rewrite: bool=False) -> bool:
    ''' save object to pickle dump '''
    if not os.path.exists(path) or rewrite:
        with open(path, 'wb') as writer:
            pickle.dump(obj, writer)
            return True
    return False


def recreate_dump(force: bool=False) -> None:
    ''' create new dump files '''
    if not os.path.exists(DUMPS_PATH) or force:
        from .raw_schema import (REQUEST_SCHEMA, RESPONSE_SCHEMA,
                                 RUZ_API_ENDPOINTS)

        if not os.path.exists(DUMPS_PATH):
            os.makedirs(DUMPS_PATH)
        save_dump(REQ_DUMP_PATH, REQUEST_SCHEMA, rewrite=True)
        save_dump(RESP_DUMP_PATH, RESPONSE_SCHEMA, rewrite=True)
        save_dump(ENDPOINTS_DUMP_PATH, RUZ_API_ENDPOINTS, rewrite=True)


DUMPS_PATH = os.path.join(os.path.dirname(__file__), "dumps")
REQ_DUMP_PATH = os.path.join(DUMPS_PATH, "REQUEST_SCHEMA.dump")
RESP_DUMP_PATH = os.path.join(DUMPS_PATH, "RESPONSE_SCHEMA.dump")
ENDPOINTS_DUMP_PATH = os.path.join(DUMPS_PATH, "RUZ_API_ENDPOINTS.dump")


recreate_dump()


REQUEST_SCHEMA = load_dump(REQ_DUMP_PATH)
assert REQUEST_SCHEMA

RESPONSE_SCHEMA = load_dump(RESP_DUMP_PATH)
assert RESPONSE_SCHEMA

RUZ_API_ENDPOINTS = load_dump(ENDPOINTS_DUMP_PATH)
assert RUZ_API_ENDPOINTS


__all__ = ("REQUEST_SCHEMA", "RESPONSE_SCHEMA", "RUZ_API_ENDPOINTS")


if __name__ == "__main__":
    recreate_dump(force=True)



'''
    Python wrapper for HSE RUZ API.

    Usage:
        from ruz import RUZ

        api = RUZ()
        assert api.v == 1
        assert api.schedule("mymail@edu.hse.ru")
'''

import os

from ruz.RUZ import RUZ
from ruz.utils import REQUEST_SCHEMA, RESPONSE_SCHEMA, EMAIL_DOMAINS

__author__ = "hell03end"
__version__ = (0, 2, 1)
__all__ = ("RUZ", "REQUEST_SCHEMA", "RESPONSE_SCHEMA")



LEVELS = {
    "NOTSET": 0,
    "DEBUG": 10,
    "INFO": 20,
    "WARN": 30,
    "WARNING": 30,
    "ERROR": 40,
    "FATAL": 50,
    "CRITICAL": 50
}


def Logger(name: str, level: int=LEVELS["DEBUG"], **kwargs) -> ...:
    import logging
    logging.basicConfig(
        format=kwargs.get(
            "format",
            "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
        ),
        level=level
    )
    return logging.getLogger(name)



import json
import re
from collections import Iterable
from datetime import datetime as dt
from datetime import timedelta as td
from functools import lru_cache
from http.client import HTTPResponse
from urllib import error, parse, request

from ruz.utils import (EMAIL_DOMAINS, EMAIL_PATTERN, REQUEST_SCHEMA,
                       RUZ_API_ENDPOINTS, RUZ_API_URL, Logger)


class RUZ(object):
    '''
        Handler for RUZ API

        All methods are transformed from CamelCase to _ notation.
    '''

    def __init__(self, strict_v1: bool=True, **kwargs):
        '''
            >>> RUZ(base_url=None)
            Traceback (most recent call last):
                ...
            PermissionError: Can't get base url!
            >>> RUZ(strict_v1=False)._url2[-2]
            '2'
            >>> RUZ()._url2 == RUZ()._url
            True
        '''
        self._url = kwargs.pop('base_url', RUZ_API_URL)
        if not self._url:
            raise PermissionError("Can't get base url!")
        self._endpoints = kwargs.pop('endpoints', RUZ_API_ENDPOINTS)
        self._url2 = self._url
        self._schema = kwargs.pop('schema', REQUEST_SCHEMA)
        self._logger = Logger(str(self.__class__))
        if not strict_v1:
            self._url2 += r"v2/"
        super(RUZ, self).__init__()

    @property
    def schema(self) -> dict:
        ''' Current request schema. '''
        return self._schema

    @property
    def v(self) -> int:
        '''
            Max API version

            >>> RUZ().v
            1
            >>> RUZ(strict_v1=False).v
            2
        '''
        return 2 if self._url2[-2] == "2" else 1

    @property
    def email_domains(self) -> tuple:
        ''' Allowed email domains '''
        return EMAIL_DOMAINS

    def _make_url(self, endpoint: str, data: dict=None, v: int=1) -> str:
        ''' Creates full url for API requests '''
        url = self._url if v == 1 or self.v == 1 else self._url2
        if data:
            return "{}{}?{}".format(url, self._endpoints[endpoint],
                                    parse.urlencode(data))
        return "{}{}".format(url, self._endpoints[endpoint])

    def _request(self, endpoint: str, data: dict=None) -> HTTPResponse:
        ''' Implements request to API with given params '''
        if self.v == 2:
            try:
                return request.urlopen(self._make_url(endpoint, data, v=2))
            except error.HTTPError as excinfo:
                self._logger.warning("v2 API unavailable: %s", excinfo)
        return request.urlopen(self._make_url(endpoint, data))

    def _verify_schema(self, endpoint: str, **params) -> None:
        '''
            Check params fit schema for certain endpoint

            >>> RUZ()._verify_schema("")
            Traceback (most recent call last):
                ...
            ValueError: Wrong endpoint: ''
        '''
        schema = self._schema.get(endpoint)
        if schema is None:
            raise KeyError("Wrong endpoint: '{}'".format(endpoint))
        for key, value in params.items():
            if key not in schema:
                raise KeyError("Wrong param '{}' for {} endpoint".format(
                    key, endpoint
                ))
            if not isinstance(value, schema[key]):
                raise ValueError("Expected {} for {} got: {}".format(
                    schema[key], key, type(value)
                ))

    def check_email(self, email: str, pattern: str=EMAIL_PATTERN) -> None:
        '''
            Check email is valid HSE corp. email.

            >>> RUZ().check_email("somemail@hse.com")
            Traceback (most recent call last):
                ...
            ValueError: Wrong email address: somemail@hse.com
            >>> not RUZ().check_email("somemail@edu.hse.ru")
            True
            >>> RUZ().check_email("somem@il@edu.hse.ru")
            Traceback (most recent call last):
                ...
            ValueError: Wrong email address: somem@il@edu.hse.ru
            >>> RUZ().check_email("somemail@google.ru")
            Traceback (most recent call last):
                ...
            ValueError: Wrong email domain: google.ru
        '''
        if not re.match(pattern, email):
            raise ValueError("Wrong email address: {}".format(email))

        domain = email.split('@')[-1]
        if domain not in self.email_domains:
            raise ValueError("Wrong email domain: {}".format(domain))

    def _verify_email(self, email: str, receiver_type: int=3) -> None:
        '''
            Check email is valid for given receiver type (to use in API)

            >>> RUZ()._verify_email("somemail@hse.com")
            Traceback (most recent call last):
                ...
            ValueError: Wrong email address: somemail@hse.com
            >>> not RUZ()._verify_email("somemail@edu.hse.ru")
            True
            >>> RUZ()._verify_email("somem@il@edu.hse.ru")
            Traceback (most recent call last):
                ...
            ValueError: Wrong email address: somem@il@edu.hse.ru
            >>> RUZ()._verify_email("somemail@google.ru")
            Traceback (most recent call last):
                ...
            ValueError: Wrong email domain: google.ru
            >>> RUZ()._verify_email("somemail@hse.ru", -1)
            Traceback (most recent call last):
                ...
            ValueError: Wrong receiverType: -1
            >>> RUZ()._verify_email("somemail@hse.ru", 2)
            Traceback (most recent call last):
                ...
            ValueError: No email needed for receiverType: 2
        '''
        self.check_email(email)

        domain = email.split('@')[-1]
        if receiver_type == 1:
            if domain != "hse.ru":
                self._logger.warning("Wrong domain for teacher: %s", domain)
        elif receiver_type == 2:
            raise ValueError("No email needed for receiverType: 2")
        elif receiver_type == 3:
            if domain != "edu.hse.ru":
                self._logger.warning("Wrong domain for student: %s", domain)
        else:
            raise ValueError("Wrong receiverType: {}".format(receiver_type))

    def _get(self, endpoint: str, **params) -> dict:
        ''' Return requested data in JSON '''
        email = params.get('email')
        if email:
            self._verify_email(email, params.get('receiverType', 3))
        self._verify_schema(endpoint, **params)
        try:
            response = self._request(endpoint, data=params)
            return json.loads(response.read().decode('utf-8'))
        except error.HTTPError as excinfo:
            self._logger.error(excinfo)
            return {}

    def schedule(self, email: str=None,
                 from_date: str=str(dt.now()).replace('-', '.')[:10],
                 to_date: str=str(dt.now() + td(days=6)).replace('-',
                                                                 '.')[:10],
                 **params) -> dict:
        '''
            Return classes schedule.

            :param from_date: str, required. Start of the period YYYY.MM.DD.
            :param to_date: str, required. End of the period YYYY.MM.DD.
            :param receiverType: int. Type of the schedule
                (1/2/3 for teacher/auditorium/student).
            :param groupOid: int. ID of group.
            :param lecturerOid: int. ID of teacher.
            :param auditoriumOid: int. ID of auditorium.
            :param studentOid: int. ID of student.
            :param email: str. Email on hse.ru (edu.hse.ru for students).

            One of the followed required: lecturerOid, groupOid, auditoriumOid,
                studentOid, email.
        '''
        return self._get(
            "schedule",
            fromDate=params.pop('fromDate', from_date),
            toDate=params.pop('toDate', to_date),
            email=email,
            **params
        )

    def schedules(self, emails: Iterable, **params) -> map:
        '''
            Classes schedule for multiply students

            >>> RUZ().schedules(123)
            Traceback (most recent call last):
                ...
            ValueError: Expect Iterable, got: <class 'int'>
        '''
        if isinstance(emails, str):
            emails = [emails]
        elif not isinstance(emails, (set, list, tuple)):
            raise ValueError("Expect Iterable, got: {}".format(type(emails)))
        return map(lambda email, params=params: self.schedule(email, **params),
                   emails)

    @lru_cache(maxsize=16)
    def groups(self, **params) -> dict:
        '''
            Return list of groups.

            :param facultyOid: int. Course ID.
            :param findText: str. Text to find.
        '''
        return self._get("groups", **params)

    @lru_cache(maxsize=16)
    def staff_of_group(self, group_id: int, **params) -> dict:
        '''
            Return list of students in group.

            :param group_id: int, required. Group' ID.
            :param findText: str. Text to find.
        '''
        return self._get("staffOfGroup", groupOid=group_id, **params)

    @lru_cache(maxsize=16)
    def streams(self, **params) -> dict:
        '''
            Return list of study streams.

            :param findText: str. Text to find.
        '''
        return self._get("streams", **params)

    @lru_cache(maxsize=16)
    def staff_of_streams(self, stream_id: int, **params) -> dict:
        '''
            Return list of the groups on study stream.

            :param stream_id: int, required. Group' ID.
        '''
        return self._get("staffOfStreams", streamOid=stream_id, **params)

    @lru_cache(maxsize=16)
    def lecturers(self, **params) -> dict:
        '''
            Return list of teachers.

            :param chairOid: int. ID of department.
            :param findText: str. Text to find.
        '''
        return self._get("lecturers", **params)

    @lru_cache(maxsize=16)
    def auditoriums(self, **params) -> dict:
        '''
            Return list of auditoriums.

            :param buildingOid: int. ID of building.
            :param findText: str. Text to find.
        '''
        return self._get("auditoriums", **params)

    @lru_cache(maxsize=1)
    def type_of_auditoriums(self) -> dict:
        ''' Return list of auditoriums' types. '''
        return self._get("typeOfAuditoriums")

    @lru_cache(maxsize=1)
    def kind_of_works(self) -> dict:
        ''' Return list of classes' types. '''
        return self._get("kindOfWorks")

    @lru_cache(maxsize=16)
    def buildings(self, **params) -> dict:
        '''
            Return list of buildings.

            :param findText: str. Text to find.
        '''
        return self._get("buildings", **params)

    @lru_cache(maxsize=16)
    def faculties(self, **params) -> dict:
        '''
            Return list of learning programs (faculties).

            :param findText: str. Text to find.
        '''
        return self._get("faculties", **params)

    @lru_cache(maxsize=16)
    def chairs(self, **params) -> dict:
        '''
            Return list of departments.

            :param facultyOid: int. ID of course (learning program).
            :param findText: str. Text to find.
        '''
        return self._get("chairs", **params)

    @lru_cache(maxsize=16)
    def sub_groups(self, **params) -> dict:
        '''
            Return list of subgroups.

            :param findText: str. Text to find.
        '''
        return self._get("subGroups", **params)


if __name__ == "__main__":
    import doctest
    doctest.testmod()
